/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  AlgoInterface.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@fr.st, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */

package merja.associations;

import merja.*;
import merja.associations.*;
import merja.data.*;
import merja.associations.itemFreq.GeneItems;
import javax.swing.JFrame;
import merja.associations.regles.GeneRegles;
import merja.gui.*;
import java.util.Vector;

/** Classe permettant de commander les algos de recherches de itemset frquent et
 * des rgles d'association
 */
public class AlgoInterface {
    private GeneItems gen;
    private GeneRegles geneRegles;
    private Hash hash;
    private int supportMin;
    private double confianceMin;
    private TransactionsReader tr;
    private Logger log;
    
    /**
     * Constructeur vide
     * => initialiser les valeurs par les mthodes set...
     * */
    public AlgoInterface(){
    }
    
    /**
     * Construteur complet
     * @param transReader  : le lecteur de fichier ou de base de donnes
     * @param supportMin   : support minimun pour le calcul des items frquents
     * @param confianceMin : confiance minimun pour qu'une rgle soit valable
     */
    public AlgoInterface(TransactionsReader transReader,int supportMin,double confianceMin){
        this();
        hash=new Hash(transReader.getAttributes());
        this.tr=transReader;
        this.supportMin=supportMin;
        this.confianceMin=confianceMin;
    }
    
    /**
     * Construteur
     * @param transReader : le lecteur de fichier ou de base de donnes
     * @param logger : panel permettant de "loguer" des infos
     */
    public AlgoInterface(TransactionsReader transReader, Logger logger){
        this();
        hash=new Hash(transReader.getAttributes());
        this.tr=transReader;
        this.supportMin=1;
        this.confianceMin=1.0;
        this.log = logger;
    }
    
    
    /**
     * Change le log
     * @param lo : nouveau log
     */
    public void setLog(Logger lo) {
        this.log = lo;
    }
    
    /**
     * Spcifice le support minimun
     * @param support : support minimun pour le calcul des items frquents
     * => r-initialise les items frquents et donc aussi les rgles mais ne les recalcule pas
     *
     */
    public void setSupportMin(int support){
        this.supportMin=support;
        this.gen=null;
        this.geneRegles=null;
    }
    
    /**
     * Spcifice le support minimun
     * @param conf : confiance minimun pour qu'une rgle soit valable
     * => ne re-initialise rien  et ne  recalcule rien
     */
    public void setConfianceMin(double conf){
        this.confianceMin=conf;
    }
    
    /**
     * Genere les items frquents par rapport au supportMin et au transReader
     * Attention ces valeurs doivent tre initialiser sinon ben... a marche pas
     */
    public void geneItemFreq(){
        gen= new GeneItems(hash,supportMin);
        Item[] trans;
        Transaction transaction = new Transaction(hash);
        
        int j;
        j=0;
        do{
            tr.goToFirstTransaction();
            trans=tr.nextTransaction();
            while(trans!=null){
                
                transaction.addItemList(trans);
                
                gen.calculSupport(transaction);
                trans=tr.nextTransaction();
            }
            //System.out.println("### "+gen.nbNoeud+" "+GeneItems.nbIdItemSupport);
            
            gen.removes();
            log.logMessage("Itration : "+j);
            //System.out.println(gen.nbNoeud+" "+GeneItems.nbIdItemSupport+" ###");
            j++;
            log.logMessage("Items frquents : "+gen.nbIdItemSupport);
        }while(gen.jointure());
    }
    
    /**
     * Genere les  par rapport au confianceMin et aux items frquents
     * Attention ces valeurs doivent tre initialiser sinon ben... a marche pas
     */
    public void geneRegle(){
        geneRegles = new GeneRegles(gen,hash,confianceMin);
        
        geneRegles.createRegle(log);
        log.logMessage("Rgles valides : "+geneRegles.getNbReglesValides());
        
    }
    
    /**
     * Recalcule les rgles si la confianceMin a t change
     */
    public void recalculateRegle(){
        
        if (geneRegles!=null){
            geneRegles.setMinConf(this.confianceMin);
            log.logMessage("Rgles : "+geneRegles.getNbRegles());
            log.logMessage("Rgles valides : "+geneRegles.getNbReglesValides());
            
        }
    }
    
    /**
     * Obsoleste
     * @return : les tous les items frquents valides sous forme de String
     */
    public String getItemsFreq(){
        if (gen!=null)
            return gen.toString();
        else
            return "";
    }
    
    /** Retourne la liste des itemset frquents
     * @return tous les itemsets dans un Vector d'<CODE>ItemSet</CODE>
     */
    public Vector getItemSetFreq(){
        if (gen!=null)
            return gen.getItemsSetFrequents();
        return null;
    }
    /** Retourne la liste des rgles d'association
     * @return tous les rgles dans un Vector de <CODE>RegleSet</CODE>
     */
    public Vector getRegleSet(){
        if (geneRegles!=null){
            return geneRegles.getRegleSet();
        }
        return null;
    }
    
    /** Retourne les rgles d'association trouves
     * @return : toutes les rgles valides sous forme de String
     */
    public String getRegle(){
        if (geneRegles!=null)
            return geneRegles.toString();
        else
            return "";
    }
    
    /**
     * Spcifice le lecteur de fichier
     * @param reader
     * r-initialise les items frquents et donc aussi les rgles mais ne les recalcule pas
     */
    public void setTransactionsReader(TransactionsReader reader){
        tr=reader;
        hash=new Hash(reader.getAttributes());
        this.gen=null;
        this.geneRegles=null;
    }
    
    
    /** Main de test
     * @param args
     */    
    public static void main(String[] args) {
        TransactionsReader arff = null;
        try {
            arff =(TransactionsReader) new ArffLoader("C:/Documents and Settings/miaousss/Mes documents/tmp/data/soybean.arff");
        }
        catch (Exception e) {}
        AlgoInterface algo = new AlgoInterface(arff,500,0.998);
        algo.geneItemFreq();
        
        System.out.println(algo.gen.toString());
        
        algo.geneRegle();
        //System.out.println("''''''''''''\n"+algo.getRegle());
        
        //System.out.println(algo.geneRegles.getNbRegles());
        //System.out.println(algo.geneRegles.getNbReglesValides());
        
        //algo.geneRegles.setMinConf(1);
        //System.out.println(algo.geneRegles.getNbReglesValides());
    }
}