/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  ArffLoader.java
 *  2003-2004 Darnet Camille, Vouriot Thierry
 *  camsss@ifrance.com, yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */
package merja.data;

import java.util.*;
import java.io.*;
import merja.associations.*;

/** Chargeur de fichier au format <CODE>arff</CODE> (fichier utilis par le logiciel
 * Weka)
 */
public class ArffLoader implements TransactionsReader {
    
    private final Vector m_itemsVector = new Vector();
    private String m_name;
    private RandomAccessFile m_accessFile;
    private final Vector m_corresp = new Vector();
    private int m_nbAttributes = 0;
    private int m_nbTransactions = 0;
    private long m_dataPos = 0;
    
    /** Constructeur
     * @param filePath chemin d'accs du fichier
     * @throws IOException erreur lors du chargement du fichier
     */    
    public ArffLoader(String filePath) throws IOException {
        m_nbAttributes = 0;
        m_dataPos = 0;
        boolean relation = false;
        boolean attribute = false;
        boolean data = false;
        
        m_accessFile = new RandomAccessFile(filePath, "r");
        String line;
        StringTokenizer token;
        while ((line = m_accessFile.readLine()) != null) {
            token = new StringTokenizer(line, " {},\t");
            String tmp;
            if (token.hasMoreTokens()) {
                tmp = token.nextToken();
                
                if (tmp.equalsIgnoreCase("@relation")) {
                    relation = true;
                    m_name = token.nextToken();
                }
                else if (tmp.equalsIgnoreCase("@attribute")){
                    attribute = true;
                    String classe = token.nextToken();
                    while (token.hasMoreTokens()) {
                        m_itemsVector.addElement(new Item(classe, token.nextToken()));
                    }
                    m_itemsVector.addElement(new Item(classe, "?"));
                    m_corresp.add(m_nbAttributes, classe);
                    m_nbAttributes++;
                }
                else if (tmp.equalsIgnoreCase("@data")) {
                    m_dataPos = m_accessFile.getFilePointer();
                    data = true;
                    if (relation && attribute) {
                        calculNbTransactions();
                        return;
                    }
                    else throw new IOException("Arff format invalide");
                }
            }
        }
        throw new IOException("Arff format invalide");
    }
    
    /** Retourne l'ensemble des attributs avec leurs valeurs
     * @return tableau d'items correspondant  tous les items diffrents
     */    
    public Item[] getAttributes() {
        Object[] tab = m_itemsVector.toArray();
        Item[] items = new Item[tab.length];
        for (int i=0; i<tab.length; i++)
            items[i] = (Item)tab[i];
        return items;
    }
    
    /** Place le pointeur sur la premire transaction de la base de donnes */    
    public void goToFirstTransaction() {
        try {
            m_accessFile.seek(m_dataPos);
        }
        catch (IOException ioe) { System.out.println("ArffLoader::goToFirstTransaction : "+ioe); }
    }
    
    /** Retourne la prochaine transaction (tableau d'items)
     * @return pochaine transaction (tableau d'items)
     */   
    public Item[] nextTransaction() {
        String line = null;
        try {
            line = m_accessFile.readLine();
        }
        catch (IOException ioe) { System.out.println("ArffLoader::nextTransaction : "+ioe); }
        if (line == null) return null;
        else if (line.trim().equals("") || line.charAt(0) == '%') return nextTransaction();
        Item [] tab = new Item [m_nbAttributes];
        
        StringTokenizer token = new StringTokenizer(line, ", \t");
        for (int i=0; i<m_nbAttributes; i++) {
            if (token.hasMoreTokens())
            tab[i] = new Item((String)m_corresp.get(i), token.nextToken());
        }
        return tab;
    }
    
    /** Retourne le nombre d'attributs
     * @return nombre d'attributs
     */    
    public int getNbAttributes() {
        return m_nbAttributes;
    }
    
    /** Retourne le nom du fichier ou de la base de donnes
     * @return nom de la base de donnes
     */    
    public String getRelationName() {
        return m_name;
    }
    
    /** Main pour tester la classe
     * @param argv arguments de la ligne de commande
     */    
    public static void main(String []argv) {
        String filePath = "c:/toto.arff";
        TransactionsReader fr = null;
        try {
            fr = new ArffLoader("d:/toto.arff");
        }
        catch (Exception e) {}
        Item[] item;
        while ((item = fr.nextTransaction()) != null) {
            for (int i=0; i <item.length; i++)
                System.out.println(item[i].toString());
        }
    }
    
    private void calculNbTransactions() throws IOException {
        String line = null;
        this.m_nbTransactions = 0;
        while ((line = m_accessFile.readLine()) != null) {
            if (!(line.trim().equals("") || line.charAt(0) == '%')) m_nbTransactions++;
        }
        goToFirstTransaction();
    }
    
    /** Retourne le nombre de transactions de la base de donnes
     * @return nombre de transactions total
     */    
    public int getNbTransactions() {
        return m_nbTransactions;
    }
    
    /** Retourne le nombre d'items total
     * @return nombre d'items total
     */    
    public int getNbItems() {
        return getNbTransactions()*getNbAttributes();
    }
    
    /** Retourne le nombre d'items ayant des valeurs diffrentes
     * @return nombre d'items diffrents
     */    
    public int getNbItemsDiff() {
        return getAttributes().length;
    }
    
}

