/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *  Database.java
 *  2003-2004 Vouriot Thierry
 *  yeri@fr.st
 *  http://www.yeri.fr.st
 *
 */

package merja.db;

import java.sql.*;
import java.util.*;
import javax.swing.*;
import java.lang.*;

/** Class permettant les manipulations sur les bases de donnes
 */
public class Database {
    
    /** (Option JDBC 1) Le resultSet n'est pas " parcourir" */
    public final static int JDBC1 = 1;
    /** (Option JDBC 2) Le resultSet est  parcourir, mais il n'est pas sensible
     *  aux modifications de la base de donnes, le resultSet ne peut pas tre
     *  utilis pour mettre  jour la base de donnes
     */
    public final static int JDBC2_INSENSITIVE_RDONLY = 2;
    /** (Option JDBC 2) Le resultSet est  parcourir, mais il n'est pas sensible
     *  aux modifications de la base de donnes, le resultSet peut tre utilis
     *  pour mettre  jour la base de donnes
     */
    public final static int JDBC2_INSENSITIVE_UPDATABLE = 3;
    /** (Option JDBC 2) Le resultSet est  parcourir, et il est sensible aux
     *  modifications de la base de donnes, le resultSet ne peut pas tre
     *  utilis pour mettre  jour la base de donnes
     */
    public final static int JDBC2_SENSITIVE_RDONLY = 4;
    /** (Option JDBC 2) Le resultSet est  parcourir, et il est sensible aux
     *  modifications de la base de donnes, le resultSet peut tre utilis
     *  pour mettre  jour la base de donnes
     */
    public final static int JDBC2_SENSITIVE_UPDATABLE = 5;
    
    /** Objet Connection qui permet d'avoir des informations sur une base de donnes */
    private Connection con;
    /** Cet objet est utilis pour excuter des requtes SQL */
    private Statement st;
    /** Contient le rsultat de la dernire requte SQL */
    private ResultSet rs = null;
    /** ResultSet qui ne sera pas accessible  l'extrieur de la class */
    private ResultSet rstemp;
    /** Permet d'avoir des informations sur les rsultats du ResultSet */
    private ResultSetMetaData rsmd;
    /** 2me ResultSetMetaData qui ne sera pas accessible  l'extrieur de la class */
    private ResultSetMetaData rsmdtemp;
    /** Permet d'obtenir des informations sur la base de donnes */
    private DatabaseMetaData dm;
    
    /** Url de la base de donnes */
    private String url;
    
    /** Constructeur */
    public Database() { }
    
    /** Constructeur qui charge un pilote JDBC donn en paramtre
     * @param driver driver JDBC
     * @throws ClassNotFoundException class du driver introuvable
     */
    public Database(String driver) throws ClassNotFoundException {
        Class.forName(driver);
    }
    
    /** Retourne l'URL de la base de donnes
     *  @return url de la bdd
     */
    public String getURL() {
        return url;
    }
    
    /** Charge un pilote JDBC donn en paramtre
     * @param driver driver JDBC
     * @throws ClassNotFoundException class du driver introuvable
     */
    public static void loadDriver(String driver) throws ClassNotFoundException {
        Class.forName(driver);
    }
    
    /** Retourne le ResultSet pour pouvoir le manipuler dans d'autres classes,
     *  attention l'objet retourn est une rfrence (pointeur) au RecordSet
     *  @return le ResultSet contenant le rsultat de la dernire requte SQL
     */
    public ResultSet getResultSet() {
        return rs;
    }
    
    /** Retourne le ResultSetMetaData
     *  @return le ResultSetMetaData contenant des informations sur le ResultSet
     */
    public ResultSetMetaData getResultSetMetaData() {
        return rsmd;
    }
    
    /** Permet de se connecter  une base spcifique
     * @param url1 URL de la base de donne
     * @param user nom de l'utilisateur de la base de donne
     * @param pass password de la base de donne
     * @param type constantes JDBC
     * @throws SQLException
     */
    public void connect(String url1, String user, String pass, int type) throws SQLException {
        url = url1;
        
        this.con = DriverManager.getConnection(url,user,pass);
        this.dm = this.con.getMetaData();
        switch (type) {
            case 1: this.st = con.createStatement();break;
            case 2: this.st = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_READ_ONLY);break;
            case 3: this.st = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE,ResultSet.CONCUR_UPDATABLE);break;
            case 4: this.st = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_READ_ONLY);break;
            case 5: this.st = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_UPDATABLE);break;
        }
    }
    
    /** Ferme la Connection et le Statement de la base de donne */
    public void disconnect() {
        try {
            if (this.rs != null)
                this.rs.close();
            if (this.rstemp != null)
                this.rstemp.close();
            if (this.con != null && !con.isClosed())
                this.con.close();
            if (this.st != null)
                this.st.close();
            
            rs = null;
            rstemp = null;
            con = null;
            st = null;
        }
        catch(SQLException e) {
            System.out.println("Database::disconnect : "+e);
        }
    }
    
    /** Teste si l'objet est toujours connect
     * @return vrai si toujours connect
     */
    public boolean isConnect() {
        return (con != null);
    }
    
    /** Ferme le RecordSet
     */
    public void closeRs() {
        try {
            if (rs!=null)
                this.rs.close();
        }
        catch(SQLException e) {
            System.out.println("Database::closeRs : "+e);
        }
    }
    
    /** Retourne la liste de toutes les tables de la base de donnes
     * @return liste des tables
     */
    public String[] getTablesNames() {
        String[] tbnames = null;
        Vector tname = new Vector();
        try{
            rstemp = dm.getTables(null,null,null,new String[] {"TABLE"});
            while (rstemp.next())
                tname.add(rstemp.getString(3));
            rstemp.close();
        }
        catch(SQLException e){
            System.out.println("Database::getTablesNames : "+e);
        }
        
        tbnames = new String[tname.size()];
        Object [] tmp = new Object[tname.size()];
        
        tmp = tname.toArray();
        for (int i=0; i < tmp.length; i++)
            tbnames[i] = (String)tmp[i];
        
        return tbnames;
    }
    
    /** Retourne pour une table la liste contenant pour chaque ligne un attributs avec son type.
     * @param tableName nom de la table
     * @return table des attributs
     */
    public String[][] getAttributes(String tableName) {
        String[][] attributs = null;
        try {
            String query = "SELECT * FROM `"+tableName+"`";
            
            this.rstemp = this.st.executeQuery(query);
            this.rsmdtemp = this.rstemp.getMetaData();
            int nb = rsmdtemp.getColumnCount();
            attributs = new String[nb][2];
            for (int i=1; i<=nb; i++) {
                attributs[i-1][0] = rsmdtemp.getColumnName(i);
                attributs[i-1][1] = rsmdtemp.getColumnTypeName(i);
            }
            if (this.rstemp != null)
                this.rstemp.close();
        }
        catch(SQLException e) {
            System.out.println("Database::getAttributs : "+e);
        }
        
        return attributs;
    }
    
    /** Effectue un "SELECT *" d'une table et affiche le rsultat sur la sortie standard
     * @param tableName nom de la table
     */
    public void showTable(String tableName) {
        try {
            String query = "SELECT * FROM `"+tableName+"`";
            this.rstemp = this.st.executeQuery(query);
            this.rsmdtemp = this.rstemp.getMetaData();
            int columnCount = this.rsmdtemp.getColumnCount();
            
            while (rstemp.next()) {
                for (int i=1;i<=columnCount;i++) {
                    if (i>1) System.out.print("  ");
                    System.out.print(rstemp.getString(i));
                }
                System.out.println();
            }
            if (this.rstemp != null)
                this.rstemp.close();
        }
        catch(SQLException e) {
            System.out.println("Database::showTable : "+e);
        }
    }
    
    /** Affiche le contenu du ResultSet sur la sortie standard */
    public void showResultSet() {
        try {
            int columnCount = this.rsmd.getColumnCount();
            while (this.rs.next()) {
                for (int i=1;i<=columnCount;i++) {
                    if (i>1) System.out.print("  ");
                    System.out.print(this.rs.getString(i));
                }
                System.out.println();
            }
            //replace le Recordset au premier enregistrement
            rs.first();
        }
        catch(SQLException e) {
            System.out.println("Database::showResultSet : "+e);
        }
    }
    
    /** Execute une requte SQL de type SELECT et actualise l'attribut rs et rsmd,
     *  le rsultat de la requte est donc dans le RecordSet rs
     *  @param sql requte SQL
     */
    public void executeSQLQueryNoEx(String sql) {
        try {
            this.rs = this.st.executeQuery(sql);
            this.rsmd = this.rs.getMetaData();
        }
        catch(SQLException e) {
            System.out.println("Database::executeSQLQueryNoEx : "+e);
        }
    }
    
    /** Execute une requte SQL de type SELECT et actualise l'attribut rs et rsmd,
     *  le rsultat de la requte est donc dans le RecordSet rs
     * @param sql requte SQL
     * @throws SQLException
     */
    public void executeSQLQuery(String sql) throws SQLException {
        this.rs = this.st.executeQuery(sql);
        this.rsmd = this.rs.getMetaData();
    }
    
    /** Execute une requte SQL de type INSERT, UPDATE, DELETE ou CREATE TABLE
     *  @param sql requte SQL
     *  @return nombre de lignes affectes par la requte
     */
    public int executeSQLUpdateNoEx(String sql) {
        int n = 0;
        try {
            n = this.st.executeUpdate(sql);
        }
        catch(SQLException e) {
            System.out.println("Database::executeSQLUpdateNoEx : "+e);
        }
        return n;
    }
    
    /** Execute une requte SQL de type INSERT, UPDATE, DELETE ou CREATE TABLE
     * @return nombre de lignes affectes par la requte
     * @param sql requte SQL
     * @throws SQLException
     */
    public int executeSQLUpdate(String sql) throws SQLException {
        return this.st.executeUpdate(sql);
    }
    
    /** Renvoie un numro non utilis dans une colonne d'une table
     *  @param tableName nom de la table
     *  @param col colonne de la table
     *  @return numro unique
     */
    public int getUniqueInt(String tableName, int col) {
        int n=0;
        try {
            boolean suite = true;
            String query = "SELECT * FROM `" + tableName + "` ORDER BY `" + col + "`";
            this.rstemp = this.st.executeQuery(query);
            while (rstemp.next() && suite) {
                if (rstemp.getInt(col) != n) {
                    suite=false;
                }
                else n++;
            }
            if (this.rstemp != null)
                this.rstemp.close();
        }
        catch(Exception e) {
            System.out.println("Database::getUniqueInt : "+e);
        }
        
        return n;
    }
}



